@@ -20,8 +20,7 @@ Parameters::Parameters() {//initalize parameters info
     inOut = new InOutStreams;
 
     //versions
-    parArray.push_back(new ParameterInfoScalar <uint> (-1, -1, "versionSTAR", &versionSTAR));
-    parArray.push_back(new ParameterInfoVector <uint> (-1, -1, "versionGenome", &versionGenome));
+    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "versionGenome", &versionGenome));
 
     //parameters
     parArray.push_back(new ParameterInfoVector <string> (-1, 2, "parametersFiles", &parametersFiles));
@@ -52,10 +51,11 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readFilesIn", &readFilesIn));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "readFilesPrefix", &readFilesPrefix));
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readFilesCommand", &readFilesCommand));
-    
+
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "readMatesLengthsIn", &readMatesLengthsIn));
     parArray.push_back(new ParameterInfoScalar <uint> (-1, -1, "readMapNumber", &readMapNumber));
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readNameSeparator", &readNameSeparator));
+    //parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "readStrand", &pReads.strandString));
 
 
     //input from BAM
@@ -73,7 +73,7 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitOutSJoneRead", &limitOutSJoneRead));
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitBAMsortRAM", &limitBAMsortRAM));
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitSjdbInsertNsj", &limitSjdbInsertNsj));
-
+    parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitNreadsSoft", &limitNreadsSoft));
 
     //output
     parArray.push_back(new ParameterInfoScalar <string>     (-1, 2, "outFileNamePrefix", &outFileNamePrefix));
@@ -87,7 +87,7 @@ Parameters::Parameters() {//initalize parameters info
     //outSAM
     parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "outSAMtype", &outSAMtype));
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outSAMmode", &outSAMmode));
-    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outSAMstrandField", &outSAMstrandField));
+    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outSAMstrandField", &outSAMstrandField.in));
     parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "outSAMattributes", &outSAMattributes));
     parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "outSAMunmapped", &outSAMunmapped.mode));
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outSAMorder", &outSAMorder));
@@ -194,15 +194,15 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignWindowsPerReadNmax", &alignWindowsPerReadNmax));
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignTranscriptsPerWindowNmax", &alignTranscriptsPerWindowNmax));
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignEndsType", &alignEndsType.in));
-    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignSoftClipAtReferenceEnds", &alignSoftClipAtReferenceEnds));
+    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignSoftClipAtReferenceEnds", &alignSoftClipAtReferenceEnds.in));
 
     parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "alignEndsProtrude", &alignEndsProtrude.in));
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignInsertionFlush", &alignInsertionFlush.in));
 
     //peOverlap
-    parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "peOverlapNbasesMin", &peOverlap.NbasesMin));    
+    parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "peOverlapNbasesMin", &peOverlap.NbasesMin));
     parArray.push_back(new ParameterInfoScalar <double>     (-1, -1, "peOverlapMMp", &peOverlap.MMp));
-    
+
     //chimeric
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "chimSegmentMin", &pCh.segmentMin));
     parArray.push_back(new ParameterInfoScalar <int>        (-1, -1, "chimScoreMin", &pCh.scoreMin));
@@ -223,21 +223,24 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "sjdbFileChrStartEnd", &pGe.sjdbFileChrStartEnd));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbGTFfile", &pGe.sjdbGTFfile));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbGTFchrPrefix", &pGe.sjdbGTFchrPrefix));
+    
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbGTFfeatureExon", &pGe.sjdbGTFfeatureExon));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbGTFtagExonParentTranscript", &pGe.sjdbGTFtagExonParentTranscript));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbGTFtagExonParentGene", &pGe.sjdbGTFtagExonParentGene));
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "sjdbGTFtagExonParentGeneName", &pGe.sjdbGTFtagExonParentGeneName));
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "sjdbGTFtagExonParentGeneType", &pGe.sjdbGTFtagExonParentGeneType));
+
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "sjdbOverhang", &pGe.sjdbOverhang));
     pGe.sjdbOverhang_par=parArray.size()-1;
     parArray.push_back(new ParameterInfoScalar <int>    (-1, -1, "sjdbScore", &pGe.sjdbScore));
-    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbInsertSave", &pGe.sjdbInsertSave)); 
-    
+    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "sjdbInsertSave", &pGe.sjdbInsertSave));
+
     //variation
-    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "varVCFfile", &var.vcfFile)); 
+    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "varVCFfile", &var.vcfFile));
 
     //WASP
-    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "waspOutputMode", &wasp.outputMode)); 
+    parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "waspOutputMode", &wasp.outputMode));
 
-    
     //quant
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "quantMode", &quant.mode));
     parArray.push_back(new ParameterInfoScalar <int>     (-1, -1, "quantTranscriptomeBAMcompression", &quant.trSAM.bamCompression));
@@ -248,10 +251,18 @@ Parameters::Parameters() {//initalize parameters info
     twoPass.pass1readsN_par=parArray.size()-1;
     parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "twopassMode", &twoPass.mode));
 
-
-//     //SW parameters
-//     parArray.push_back(new ParameterInfoScalar <uint> (-1, -1, "swMode", &swMode));
-//     parArray.push_back(new ParameterInfoScalar <uint> (-1, -1, "swWinCoverageMinP", &swWinCoverageMinP));
+    //solo
+    parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloType", &pSolo.typeStr));
+    parArray.push_back(new ParameterInfoScalar <uint32>   (-1, -1, "soloCBstart", &pSolo.cbS));
+    parArray.push_back(new ParameterInfoScalar <uint32>   (-1, -1, "soloUMIstart", &pSolo.umiS));
+    parArray.push_back(new ParameterInfoScalar <uint32>   (-1, -1, "soloCBlen", &pSolo.cbL));
+    parArray.push_back(new ParameterInfoScalar <uint32>   (-1, -1, "soloUMIlen", &pSolo.umiL));
+    parArray.push_back(new ParameterInfoScalar <uint32>   (-1, -1, "soloBarcodeReadLength", &pSolo.bL));
+    parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloCBwhitelist", &pSolo.soloCBwhitelist));
+    parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloStrand", &pSolo.strandStr));
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "soloOutFileNames", &pSolo.outFileNames));
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "soloFeatures", &pSolo.featureIn));
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "soloUMIdedup", &pSolo.umiDedup));
 
     parameterInputName.push_back("Default");
     parameterInputName.push_back("Command-Line-Initial");
@@ -329,10 +340,11 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
-    inOut->logMain << "STAR version=" << STAR_VERSION << "\n"<<flush;
-    inOut->logMain << "STAR compilation time,server,dir=" << COMPILATION_TIME_PLACE << "\n"<<flush;
-
-
+    inOut->logMain << "STAR version=" << STAR_VERSION << "\n";
+    inOut->logMain << "STAR compilation time,server,dir=" << COMPILATION_TIME_PLACE << "\n";
+    #ifdef COMPILE_FOR_LONG_READS
+           inOut->logMain << "Compiled for LONG reads" << "\n";
+    #endif
 
     //define what goes to cout
     if (outStd=="Log") {
@@ -418,12 +430,6 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
 
 
     ///////////////////////////////////////// Old variables
-    outputBED[0]=0; outputBED[1]=0; //controls the format of starBED output. Will be replaced with HDF5 output
-
-    //annot scoring
-    annotScoreScale=0;
-    annotSignalFile="-";
-
     //splitting
     Qsplit=0;
     maxNsplit=10;
@@ -440,7 +446,6 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         runDirPerm=S_IRWXU;
     } else if (runDirPermIn=="All_RWX")
     {
-//         umask(0); //this should be defined by the user!
         runDirPerm= S_IRWXU | S_IRWXG | S_IRWXO;
     } else
     {
@@ -520,7 +525,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
-    
+
     //remove duplicates parameters
     if (removeDuplicates.mode=="UniqueIdentical")
     {
@@ -535,9 +540,9 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             ostringstream errOut;
             errOut << "EXITING because of fatal PARAMETERS error: unrecognized option in of --bamRemoveDuplicatesType="<<removeDuplicates.mode<<"\n";
             errOut << "SOLUTION: use allowed option: - or UniqueIdentical or UniqueIdenticalNotMulti";
-            exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);        
-    };    
-    
+            exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
+
     if (runMode=="alignReads") {
         inOut->logProgress.open((outFileNamePrefix + "Log.progress.out").c_str());
     } else if (runMode=="inputAlignmentsFromBAM") {
@@ -648,7 +653,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     };
 
     //versions
-    for (uint ii=0;ii<2;ii++) {
+    for (uint ii=0;ii<1;ii++) {
         if (parArray[ii]->inputLevel>0) {
             ostringstream errOut;
             errOut <<"EXITING because of fatal input ERROR: the version parameter "<< parArray[ii]->nameString << " cannot be re-defined by the user\n";
@@ -676,7 +681,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         errOut <<"EXITING: fatal input ERROR: --outSAMorder=PairedKeepInputOrder is presently only compatible with SAM output, i.e. default --outSMAtype SAM\n";
         errOut <<"SOLUTION: re-run STAR without --outSAMorder=PairedKeepInputOrder, or with --outSAMorder=PairedKeepInputOrder --outSMAtype SAM .\n";
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
-    };    
+    };
     //SJ filtering
     for (int ii=0;ii<4;ii++) {
         if (outSJfilterOverhangMin.at(ii)<0) outSJfilterOverhangMin.at(ii)=numeric_limits<int32>::max();
@@ -723,7 +728,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     if (outSAMfilter.mode.at(0)=="KeepOnlyAddedReferences")
     {
         outSAMfilter.KeepOnlyAddedReferences=true;
-    } else if (outSAMfilter.mode.at(0)=="KeepAllAddedReferences")  
+    } else if (outSAMfilter.mode.at(0)=="KeepAllAddedReferences")
     {
         outSAMfilter.KeepAllAddedReferences=true;
     } else if (outSAMfilter.mode.at(0)=="None")
@@ -737,6 +742,14 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
+    if ( (outSAMfilter.KeepOnlyAddedReferences || outSAMfilter.KeepAllAddedReferences) && pGe.gFastaFiles.at(0)=="-" ) {
+        ostringstream errOut;
+        errOut <<"EXITING because of FATAL INPUT ERROR: --outSAMfilter KeepOnlyAddedReferences OR KeepAllAddedReferences options can only be used if references are added on-the-fly with --genomeFastaFiles" <<"\n";
+        errOut <<"SOLUTION: use default --outSAMfilter None, OR add references with --genomeFataFiles\n";
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
+
+
     if (outMultimapperOrder.mode=="Old_2.4")
     {
         outMultimapperOrder.random=false;
@@ -762,7 +775,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         errOut <<"SOLUTION: specify one of the allowed values: Fastx or SAM\n";
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
-        
+
     if (readFilesTypeN==1) {
         readNmates=readFilesIn.size(); //for now the number of mates is defined by the number of input files
     } else if (readFilesTypeN==10) {//find the number of mates from the SAM file
@@ -777,6 +790,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
     };
+    readNmatesIn=readNmates;
 
     //two-pass
     if (parArray.at(twoPass.pass1readsN_par)->inputLevel>0  && twoPass.mode=="None")
@@ -829,8 +843,8 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             errOut <<"SOLUTION: please check the path and writing permissions \n";
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
-    };   
-    
+    };
+
     // openReadFiles depends on twoPass for reading SAM header
     if (runMode=="alignReads" && pGe.gLoad!="Remove" && pGe.gLoad!="LoadAndExit") {//open reads files to check if they are present
         openReadsFiles();
@@ -913,14 +927,14 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             //inOut->fastaOutSeqs.open("Seqs.out.fasta");
         };
     };
-    
+
     //variation
     var.yes=false;
     if (var.vcfFile!="-")
     {
         var.yes=true;
-    };         
-        
+    };
+
     //WASP
     wasp.yes=false;
     wasp.SAMtag=false;
@@ -943,14 +957,75 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
-    
      if (wasp.yes && outSAMtype.at(0)!="BAM") {
         ostringstream errOut;
         errOut <<"EXITING because of FATAL INPUT ERROR: --waspOutputMode requires output to BAM file\n";
         errOut <<"SOLUTION: re-run STAR with --waspOutputMode ... and --outSAMtype BAM ... \n";
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
+
+    //quantification parameters
+    quant.yes=false;
+    quant.geneFull.yes=false;
+    quant.geCount.yes=false;
+    quant.trSAM.yes=false;
+    quant.trSAM.bamYes=false;
+    quant.trSAM.indel=false;
+    quant.trSAM.softClip=false;
+    quant.trSAM.singleEnd=false; 
+    if (quant.mode.at(0) != "-") {
+        quant.yes=true;
+        for (uint32 ii=0; ii<quant.mode.size(); ii++) {
+            if (quant.mode.at(ii)=="TranscriptomeSAM") {
+                quant.trSAM.yes=true;
+
+                if (quant.trSAM.bamCompression>-2)
+                    quant.trSAM.bamYes=true;
+
+                if (quant.trSAM.bamYes) {
+                    if (outStd=="BAM_Quant") {
+                        outFileNamePrefix="-";
+                    } else {
+                        outQuantBAMfileName=outFileNamePrefix + "Aligned.toTranscriptome.out.bam";
+                    };
+                    inOut->outQuantBAMfile=bgzf_open(outQuantBAMfileName.c_str(),("w"+to_string((long long) quant.trSAM.bamCompression)).c_str());
+                };
+                if (quant.trSAM.ban=="IndelSoftclipSingleend") {
+                    quant.trSAM.indel=false;
+                    quant.trSAM.softClip=false;
+                    quant.trSAM.singleEnd=false;
+                } else if (quant.trSAM.ban=="Singleend") {
+                    quant.trSAM.indel=true;
+                    quant.trSAM.softClip=true;
+                    quant.trSAM.singleEnd=false;
+                };
+            } else if  (quant.mode.at(ii)=="GeneCounts") {
+                quant.geCount.yes=true;
+                quant.geCount.outFile=outFileNamePrefix + "ReadsPerGene.out.tab";
+            } else {
+                ostringstream errOut;
+                errOut << "EXITING because of fatal INPUT error: unrecognized option in --quantMode=" << quant.mode.at(ii) << "\n";
+                errOut << "SOLUTION: use one of the allowed values of --quantMode : TranscriptomeSAM or GeneCounts or - .\n";
+                exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+            };
+        };
+    };
+
+    outSAMstrandField.type=0; //none
+    if (outSAMstrandField.in=="None") {
+        outSAMstrandField.type=0;
+    } else if (outSAMstrandField.in=="intronMotif") {
+        outSAMstrandField.type=1;
+    } else {
+        ostringstream errOut;
+        errOut << "EXITING because of fatal INPUT error: unrecognized option in outSAMstrandField=" << outSAMstrandField.in << "\n";
+        errOut << "SOLUTION: use one of the allowed values of --outSAMstrandField : None or intronMotif \n";
+        exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
     
+    //solo
+    pSolo.initialize(this);
+
     //outSAMattributes
     outSAMattrPresent.NH=false;//TODO re-write as class with constructor?
     outSAMattrPresent.HI=false;
@@ -961,14 +1036,19 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     outSAMattrPresent.jM=false;
     outSAMattrPresent.jI=false;
     outSAMattrPresent.RG=false;
-    outSAMattrPresent.MC=false;    
+    outSAMattrPresent.MC=false;
     outSAMattrPresent.XS=false;
     outSAMattrPresent.vA=false;
     outSAMattrPresent.vG=false;
     outSAMattrPresent.vW=false;
     outSAMattrPresent.ch=false;
     outSAMattrPresent.rB=false;
-            
+    outSAMattrPresent.CR=false;
+    outSAMattrPresent.CY=false;
+    outSAMattrPresent.UR=false;
+    outSAMattrPresent.UY=false;
+
+
     //for quant SAM output only NH and HI flags
     outSAMattrPresentQuant=outSAMattrPresent;
     outSAMattrPresentQuant.NH=true;
@@ -1016,10 +1096,10 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             outSAMattrPresent.vA=true;
         } else if (vAttr1.at(ii)== "vG") {
             outSAMattrOrder.push_back(ATTR_vG);
-            outSAMattrPresent.vG=true; 
+            outSAMattrPresent.vG=true;
         } else if (vAttr1.at(ii)== "vW") {
             outSAMattrOrder.push_back(ATTR_vW);
-            outSAMattrPresent.vW=true;              
+            outSAMattrPresent.vW=true;
         } else if (vAttr1.at(ii)== "RG") {
             outSAMattrOrder.push_back(ATTR_RG);
             outSAMattrOrderQuant.push_back(ATTR_RG);
@@ -1027,21 +1107,37 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         } else if (vAttr1.at(ii)== "rB") {
             outSAMattrOrder.push_back(ATTR_rB);
             outSAMattrOrderQuant.push_back(ATTR_rB);
-            outSAMattrPresent.rB=true;            
+            outSAMattrPresent.rB=true;
         } else if (vAttr1.at(ii)== "ch") {
             outSAMattrOrder.push_back(ATTR_ch);
             outSAMattrOrderQuant.push_back(ATTR_ch);
-            outSAMattrPresent.ch=true;      
+            outSAMattrPresent.ch=true;
         } else if (vAttr1.at(ii)== "MC") {
             outSAMattrOrder.push_back(ATTR_MC);
             outSAMattrOrderQuant.push_back(ATTR_MC);
-            outSAMattrPresent.MC=true;                        
+            outSAMattrPresent.MC=true;
+        } else if (vAttr1.at(ii)== "CR") {
+            outSAMattrOrder.push_back(ATTR_CR);
+            outSAMattrOrderQuant.push_back(ATTR_CR);
+            outSAMattrPresent.CR=true;
+        } else if (vAttr1.at(ii)== "CY") {
+            outSAMattrOrder.push_back(ATTR_CY);
+            outSAMattrOrderQuant.push_back(ATTR_CY);
+            outSAMattrPresent.CY=true;
+        } else if (vAttr1.at(ii)== "UR") {
+            outSAMattrOrder.push_back(ATTR_UR);
+            outSAMattrOrderQuant.push_back(ATTR_UR);
+            outSAMattrPresent.UR=true;
+        } else if (vAttr1.at(ii)== "UY") {
+            outSAMattrOrder.push_back(ATTR_UY);
+            outSAMattrOrderQuant.push_back(ATTR_UY);
+            outSAMattrPresent.UY=true;
         } else if (vAttr1.at(ii)== "XS") {
             outSAMattrOrder.push_back(ATTR_XS);
             outSAMattrPresent.XS=true;
-            if (outSAMstrandField!="intronMotif") {
+            if (outSAMstrandField.type!=1) {
                 inOut->logMain << "WARNING --outSAMattributes contains XS, therefore STAR will use --outSAMstrandField intronMotif" <<endl;
-                outSAMstrandField="intronMotif";
+                outSAMstrandField.type=1;
             };
         } else {
             ostringstream errOut;
@@ -1050,21 +1146,21 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
     };
-    
+
     if  (!var.yes && (outSAMattrPresent.vA | outSAMattrPresent.vG)) {
         ostringstream errOut;
         errOut <<"EXITING because of fatal PARAMETER error: --outSAMattributes contains vA and/or vG tag(s), but --varVCFfile is not set\n";
         errOut <<"SOLUTION: re-run STAR with a --varVCFfile option, or without vA/vG tags in --outSAMattributes\n";
-        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);        
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
-    
+
     if (!wasp.yes && outSAMattrPresent.vW) {
         ostringstream errOut;
         errOut <<"EXITING because of fatal PARAMETER error: --outSAMattributes contains vW tag, but --waspOutputMode is not set\n";
         errOut <<"SOLUTION: re-run STAR with a --waspOutputMode option, or without vW tags in --outSAMattributes\n";
-        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);        
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
-    
+
     if (outSAMattrRG.size()>0 && !outSAMattrPresent.RG) {
         outSAMattrOrder.push_back(ATTR_RG);
         outSAMattrOrderQuant.push_back(ATTR_RG);
@@ -1077,7 +1173,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
-    if (outSAMstrandField=="intronMotif" && !outSAMattrPresent.XS) {
+    if (outSAMstrandField.type==1 && !outSAMattrPresent.XS) {
         outSAMattrOrder.push_back(ATTR_XS);
         inOut->logMain << "WARNING --outSAMstrandField=intronMotif, therefore STAR will output XS attribute" <<endl;
     };
@@ -1087,8 +1183,15 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         outSAMattrOrderQuant.push_back(ATTR_vW);
         outSAMattrPresent.vW=true;
         inOut->logMain << "WARNING --waspOutputMode is set, therefore STAR will output vW attribute" <<endl;
-    };    
-    
+    };
+
+    if (pSolo.type==0 && (outSAMattrPresent.CR || outSAMattrPresent.CY || outSAMattrPresent.UR || outSAMattrPresent.UY) ) {
+            ostringstream errOut;
+            errOut <<"EXITING because of FATAL INPUT ERROR: --outSAMattributes contains CR/CY/UR/UY tags, but --soloType is not set\n";
+            errOut <<"SOLUTION: re-run STAR without these attribures, or with --soloType set\n";
+            exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
+
     //chimeric
     pCh.out.bam=false;
     pCh.out.junctions=false;
@@ -1105,7 +1208,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             } else if (type1=="HardClip") {
                 pCh.out.bamHardClip=true;
             } else if (type1=="SoftClip") {
-                pCh.out.bamHardClip=false;     
+                pCh.out.bamHardClip=false;
             } else {
                 ostringstream errOut;
                 errOut <<"EXITING because of FATAL INPUT ERROR: unknown/unimplemented value for --chimOutType: "<<type1 <<"\n";
@@ -1121,7 +1224,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
                 exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
 
-        if (pCh.multimapNmax>0 && (pCh.out.bam || pCh.out.samOld)) {       
+        if (pCh.multimapNmax>0 && (pCh.out.bam || pCh.out.samOld)) {
                 ostringstream errOut;
                 errOut <<"EXITING because of fatal PARAMETERS error: --chimMultimapNmax > 0 (new chimeric detection) presently only works with --chimOutType Junctions\n";
                 errOut <<"SOLUTION: re-run with --chimOutType Junctions\n";
@@ -1180,8 +1283,8 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         alignEndsType.ext[0][0]=true;
     } else if (alignEndsType.in=="Extend5pOfReads12" )
     {
-        alignEndsType.ext[0][0]=true;        
-        alignEndsType.ext[1][0]=true;  
+        alignEndsType.ext[0][0]=true;
+        alignEndsType.ext[1][0]=true;
     } else if (alignEndsType.in=="Extend3pOfRead1" )
     {
         alignEndsType.ext[0][1]=true;
@@ -1220,44 +1323,6 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
-    //quantification parameters
-    quant.yes=false;
-    quant.geCount.yes=false;
-    quant.trSAM.yes=false;
-    if (quant.mode.at(0) != "-") {
-        quant.yes=true;
-        for (uint32 ii=0; ii<quant.mode.size(); ii++) {
-            if (quant.mode.at(ii)=="TranscriptomeSAM") {
-                quant.trSAM.yes=true;
-                if (outStd=="BAM_Quant") {
-                    outFileNamePrefix="-";
-                } else {
-                    outQuantBAMfileName=outFileNamePrefix + "Aligned.toTranscriptome.out.bam";
-                };
-                inOut->outQuantBAMfile=bgzf_open(outQuantBAMfileName.c_str(),("w"+to_string((long long) quant.trSAM.bamCompression)).c_str());
-                if (quant.trSAM.ban=="IndelSoftclipSingleend")
-                {
-                    quant.trSAM.indel=false;
-                    quant.trSAM.softClip=false;
-                    quant.trSAM.singleEnd=false;
-                } else if (quant.trSAM.ban=="Singleend")
-                {
-                    quant.trSAM.indel=true;
-                    quant.trSAM.softClip=true;
-                    quant.trSAM.singleEnd=false;
-                };
-            } else if  (quant.mode.at(ii)=="GeneCounts") {
-                quant.geCount.yes=true;
-                quant.geCount.outFile=outFileNamePrefix + "ReadsPerGene.out.tab";
-            } else {
-                ostringstream errOut;
-                errOut << "EXITING because of fatal INPUT error: unrecognized option in --quant.mode=" << quant.mode.at(ii) << "\n";
-                errOut << "SOLUTION: use one of the allowed values of --quant.mode : TranscriptomeSAM or - .\n";
-                exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
-            };
-        };
-    };
-
    //sjdb insert on the fly
 
     sjdbInsert.pass1=false;
@@ -1335,24 +1400,24 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
 
 
     //outSAMunmapped
+    outSAMunmapped.yes=false;
+    outSAMunmapped.within=false;
+    outSAMunmapped.keepPairs=false;
     if (outSAMunmapped.mode.at(0)=="None" && outSAMunmapped.mode.size()==1) {
-        outSAMunmapped.yes=false;
-        outSAMunmapped.within=false;
-        outSAMunmapped.keepPairs=false;
+        //nothing to do, all false
     } else if (outSAMunmapped.mode.at(0)=="Within" && outSAMunmapped.mode.size()==1) {
         outSAMunmapped.yes=true;
         outSAMunmapped.within=true;
-        outSAMunmapped.keepPairs=false;   
     } else if (outSAMunmapped.mode.at(0)=="Within" && outSAMunmapped.mode.at(1)=="KeepPairs") {
         outSAMunmapped.yes=true;
         outSAMunmapped.within=true;
-        outSAMunmapped.keepPairs=true;   
+        if (readNmates==2)
+            outSAMunmapped.keepPairs=true;
     } else {
         ostringstream errOut;
         errOut << "EXITING because of fatal PARAMETERS error: unrecognized option for --outSAMunmapped=";
         for (uint ii=0; ii<outSAMunmapped.mode.size(); ii++) errOut <<" "<< outSAMunmapped.mode.at(ii);
-        errOut << "\n";
-        errOut << "SOLUTION: use allowed options: None OR Within OR Within KeepPairs";
+        errOut << "\nSOLUTION: use allowed options: None OR Within OR Within KeepPairs";
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
 
@@ -1369,7 +1434,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
     };
-    
+
     if (alignInsertionFlush.in=="None") {
         alignInsertionFlush.flushRight=false;
     } else if (alignInsertionFlush.in=="Right") {
@@ -1380,15 +1445,32 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         errOut << "SOLUTION: use allowed option: None or Right";
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
-        
-    
+
+
     //peOverlap
     if (peOverlap.NbasesMin>0) {
         peOverlap.yes=true;
     } else {
         peOverlap.yes=false;
     };
-    
+
+    //alignSoftClipAtReferenceEnds.in
+    if (alignSoftClipAtReferenceEnds.in=="Yes") {
+        alignSoftClipAtReferenceEnds.yes=true;
+    } else if (alignSoftClipAtReferenceEnds.in=="No") {
+        alignSoftClipAtReferenceEnds.yes=false;
+    } else {
+        ostringstream errOut;
+        errOut << "EXITING because of fatal PARAMETERS error: unrecognized option in of --alignSoftClipAtReferenceEnds="<<alignSoftClipAtReferenceEnds.in<<"\n";
+        errOut << "SOLUTION: use allowed option: Yes or No";
+        exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
+
+    outSAMreadIDnumber=false;
+    if (outSAMreadID=="Number") {
+        outSAMreadIDnumber=true;
+    };
+
     ////////////////////////////////////////////////
     inOut->logMain << "Finished loading and checking parameters\n" <<flush;
 };
@@ -1440,7 +1522,7 @@ int Parameters::scanOneLine (string &lineIn, int inputLevel, int inputLevelReque
 
     if (iPar==parArray.size()) {//string is not identified
         ostringstream errOut;
-        errOut << "EXITING: FATAL INPUT ERROR: unrecoginzed parameter name \""<< parIn << "\" in input \"" << parameterInputName.at(inputLevel) <<"\"\n";
+        errOut << "EXITING: FATAL INPUT ERROR: unrecognized parameter name \""<< parIn << "\" in input \"" << parameterInputName.at(inputLevel) <<"\"\n";
         errOut << "SOLUTION: use correct parameter name (check the manual)\n"<<flush;
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     } else {//found the corresponding parameter
