@@ -13,15 +13,16 @@ public:
     uint32 umiS,umiL; //umi start,length
     uint32 bL; //total barcode length
     string soloCBwhitelist;
-    std::vector <uint32> cbWL;
+    std::vector <uint64> cbWL;
+    bool cbWLyes;
     string strandStr;
     int32 strand;
     //features
     const static vector<string> featureNames;
     vector<string> featureIn;
-    vector<uint32> features;
+    vector<uint32> features, featureInd;
     uint32 nFeatures;
-    bool featureYes[2]; //which features are requested
+    bool featureYes[3]; //which features are requested
     //filtering
     char QSbase,QSmax;//quality score base and cutoff
     float cbMinP;//for CBs with non-exact matching to WL, min posterior probability
@@ -32,8 +33,8 @@ public:
     //output
     vector<string> outFileNames;
     //constants
-    uint32 umiMaskLow, umiMaskHigh; //low/high half bit-mask or UMIs 
-    
+    uint32 umiMaskLow, umiMaskHigh; //low/high half bit-mask or UMIs
+
     void initialize(Parameters *pPin);
 private:
     Parameters *pP;
