@@ -8,11 +8,10 @@ void ReadAlign::outputAlignments() {
 
     bool mateMapped[2]={false,false};
 
-    vector<int32> readGenes;
+    set<uint32> readGeneFull={},readGene={};
     vector<uint32> readTranscripts={};
-    set<uint32> readTrGenes={};
-    vector<array<uint64,2>> readSJs={};
-    
+    vector<int32> readGeneExon={};
+
     outFilterPassed=true;//only false if the alignment is held for outFilterBySJoutStage
     if (unmapType==-1) {//output transcripts
         if (P.outFilterBySJoutStage==1) {//filtering by SJout
@@ -43,7 +42,7 @@ void ReadAlign::outputAlignments() {
                 };
             };
         };
-        
+
         if (P.outSJfilterReads=="All" || nTr==1) {
             uint sjReadStartN=chunkOutSJ1->N;
             for (uint iTr=0;iTr<nTr;iTr++) {//report SJs for all transcripts
@@ -90,14 +89,13 @@ void ReadAlign::outputAlignments() {
                 errOut  << "EXITING because of a BUG: nTr=0 in outputAlignments.cpp";
                 exitWithError(errOut.str(), std::cerr, P.inOut->logMain, EXIT_CODE_BUG, P);
             };
-            
+
             nTrOut=min(P.outSAMmultNmax,nTrOut); //number of to write to SAM/BAM files
 
+            soloRead->readBar->getCBandUMI(readNameExtra.at(0));
+            
+            //write to SAM/BAM
             for (uint iTr=0;iTr<nTrOut;iTr++) {//write all transcripts
-                //mate mapped = true if a mate was present in one of the trancsripts
-                //mateMapped[trMult[iTr]->exons[0][EX_iFrag]]=true;
-                //mateMapped[trMult[iTr]->exons[trMult[iTr]->nExons-1][EX_iFrag]]=true;
-
                 //mateMapped1 = true if a mate is present in this transcript
                 bool mateMapped1[2]={false,false};
                 mateMapped1[trMult[iTr]->exons[0][EX_iFrag]]=true;
@@ -113,7 +111,7 @@ void ReadAlign::outputAlignments() {
                 if ((P.outBAMunsorted || P.outBAMcoord) && outSAMfilterYes) {//BAM output
                     alignBAM(*(trMult[iTr]), nTr, iTr, mapGen.chrStart[trMult[iTr]->Chr], (uint) -1, (uint) -1, 0, -1, NULL, P.outSAMattrOrder,outBAMoneAlign, outBAMoneAlignNbytes);
 
-                    if (P.outBAMunsorted) {//unsorted 
+                    if (P.outBAMunsorted) {//unsorted
                         for (uint imate=0; imate<P.readNmates; imate++) {//output each mate
                             outBAMunsorted->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], (imate>0 || iTr>0) ? 0 : (outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1])*2*nTrOut);
                         };
@@ -135,8 +133,8 @@ void ReadAlign::outputAlignments() {
             };
 
             mateMapped[trBest->exons[0][EX_iFrag]]=true;
-            mateMapped[trBest->exons[trBest->nExons-1][EX_iFrag]]=true;            
-            
+            mateMapped[trBest->exons[trBest->nExons-1][EX_iFrag]]=true;
+
             if (P.readNmates>1 && !(mateMapped[0] && mateMapped[1]) ) {
                 unmapType=4;
             };
@@ -161,37 +159,44 @@ void ReadAlign::outputAlignments() {
 
             if (P.outSJfilterReads=="All" || nTr==1) {
                 uint sjReadStartN=chunkOutSJ->N;
-                for (uint iTr=0;iTr<nTr;iTr++) {//write all transcripts
+                for (uint iTr=0;iTr<nTr;iTr++) {//write all transcripts junctions
                     outputTranscriptSJ (*(trMult[iTr]), nTr, chunkOutSJ, sjReadStartN);
                 };
             };
-
+            
+            //genes
             if ( P.quant.geCount.yes ) {
-                chunkTr->geneCountsAddAlign(nTr, trMult, readGenes);
+                chunkTr->geneCountsAddAlign(nTr, trMult, readGeneExon);
             };
 
-            if ( P.quant.trSAM.yes ) {//NOTE: the transcripts are changed by this function (soft-clipping extended), cannot be reused
-                quantTranscriptome(chunkTr, nTrOut, trMult,  alignTrAll, readTranscripts, readTrGenes);
+            if ( P.quant.geneFull.yes ) {
+                chunkTr->geneFullAlignOverlap(nTr, trMult, P.pSolo.strand, readGeneFull);
             };
+
+            //transcripts
+            if ( P.quant.trSAM.yes ) {//NOTE: the transcripts are changed by this function (soft-clipping extended), cannot be reused
+                quantTranscriptome(chunkTr, nTrOut, trMult,  alignTrAll, readTranscripts, readGene);
+            };
+
+            //solo
+            soloRead->record(nTr, readGene, readGeneFull, trMult[0]); 
         };
     };
-    
-    if (outFilterPassed) {//otherwise the alignment was held and will be counted at the 2nd stage
-        soloRead->record(readNameExtra.at(0), nTr, readTrGenes, trMult[0]);
-    };
 
-    if (unmapType>=0) {
+    if (unmapType>=0) {//unmapped reads
         statsRA.unmappedAll++;
+        soloRead->readBar->getCBandUMI(readNameExtra.at(0));
+        soloRead->record(nTr, readGene, readGeneFull, trMult[0]);         
     };
 
     if ( P.outSAMunmapped.within && unmapType>=0 && unmapType<4 ) {//output unmapped within && unmapped read && both mates unmapped
-        if (P.outBAMcoord || P.outBAMunsorted || P.quant.trSAM.yes) {//BAM output
+        if (P.outBAMcoord || P.outBAMunsorted || P.quant.trSAM.bamYes) {//BAM output
             alignBAM(*trBest, 0, 0, mapGen.chrStart[trBest->Chr], (uint) -1, (uint) -1, 0, unmapType, mateMapped, P.outSAMattrOrder, outBAMoneAlign, outBAMoneAlignNbytes);
             for (uint imate=0; imate<P.readNmates; imate++) {//output each mate
                 if (P.outBAMunsorted) {
                     outBAMunsorted->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], imate>0 ? 0 : outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1]);
                 };
-                if (P.quant.trSAM.yes) {
+                if (P.quant.trSAM.bamYes) {
                     outBAMquant->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], imate>0 ? 0 : outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1]);
                 };
                 if (P.outBAMcoord) {
@@ -204,10 +209,11 @@ void ReadAlign::outputAlignments() {
             outBAMbytes+= outputTranscriptSAM(*trBest, 0, 0, (uint) -1, (uint) -1, 0, unmapType, mateMapped, outSAMstream);
         };
     };
+
     if (unmapType>=0 && P.outReadsUnmapped=="Fastx" ){//output to fasta/q files
        for (uint im=0;im<P.readNmates;im++) {
            chunkOutUnmappedReadsStream[im] << readNameMates[im]  <<" "<<im<<":"<< readFilter <<": "<< readNameExtra[im];
-           if (P.readNmates>1) 
+           if (P.readNmates>1)
                chunkOutUnmappedReadsStream[im] <<" "<< int(mateMapped[0]) <<  int(mateMapped[1]);
            chunkOutUnmappedReadsStream[im] <<"\n";
            chunkOutUnmappedReadsStream[im] << Read0[im] <<"\n";
@@ -216,6 +222,12 @@ void ReadAlign::outputAlignments() {
                 chunkOutUnmappedReadsStream[im] << Qual0[im] <<"\n";
             };
        };
+       if (P.pSolo.type>0) {//need to output 2nd (barcode) read
+           chunkOutUnmappedReadsStream[1] << readNameMates[0] <<"\n";
+           uint32 qualStart = readNameExtra[0].find(' ');
+           chunkOutUnmappedReadsStream[1] << readNameExtra[0].substr(0,qualStart) <<"\n+\n";
+           chunkOutUnmappedReadsStream[1] << readNameExtra[0].substr(qualStart+1) <<"\n";
+       };
     };
 };
 
