@@ -26,44 +26,36 @@
 #include "SjdbClass.h"
 #include "sjdbInsertJunctions.h"
 #include "Variation.h"
+#include "Solo.h"
+
 #include "bam_cat.h"
 
 #include "htslib/htslib/sam.h"
 #include "parametersDefault.xxd"
 
+void usage(int usageType) {
+    cout << "Usage: STAR  [options]... --genomeDir REFERENCE   --readFilesIn R1.fq R2.fq\n";
+    cout <<"Spliced Transcripts Alignment to a Reference (c) Alexander Dobin, 2009-2019\n\n";
+    cout <<"For more details see:\n";
+    cout <<"<https://github.com/alexdobin/STAR>\n";
+    cout <<"<https://github.com/alexdobin/STAR/blob/master/doc/STARmanual.pdf>\n";
 
-
-void usage() {
-    printf("Usage: STAR  [options]... --genomeDir REFERENCE   --readFilesIn R1.fq R2.fq\n");
-    printf("Spliced Transcripts Alignment to a Reference (c) Alexander Dobin, 2009-2015\n\n");
-
-    std::cout << parametersDefault;
-
-    printf("For more details see:\n");
-    printf("<https://github.com/alexdobin/STAR>\n");
-    printf("<https://github.com/alexdobin/STAR/blob/master/doc/STARmanual.pdf>\n");
-
+    if (usageType==0) {//brief
+        cout << "\nTo list all parameters, run STAR --help\n";
+    } else {//full
+        cout <<'\n'<< parametersDefault;
+    };
     exit(0);
 }
 
 
 int main(int argInN, char* argIn[]) {
-    
-    //debug
-//     uint nn=4000000000;
-//     uint* dummy=new uint[nn];
-//     std::cout <<"Allocated"<<endl;
-//     for (uint ii=0;ii<nn;ii++) {
-//         dummy[ii]=14829735431805718528LLU;;
-//     };
-//     std::cout <<"Filled "<<dummy[0]<<endl;
-//     delete[] dummy;     
-//     std::cout <<"Deleted"<<endl;
-    
     // If no argument is given, or the first argument is either '-h' or '--help', run usage()
-    if((argInN == 1) || (argInN == 2 && (strcmp("-h",argIn[1]) == 0 || strcmp ("--help",argIn[1]) == 0 ))) {
-        usage();
-    }
+    if (argInN == 1) {
+        usage(0);
+    } else if (argInN == 2 && (strcmp("-h",argIn[1]) == 0 || strcmp ("--help",argIn[1]) == 0 )) {
+        usage(1);
+    };
 
     time(&g_statsAll.timeStart);
 
@@ -73,17 +65,17 @@ int main(int argInN, char* argIn[]) {
     *(P.inOut->logStdOut) << timeMonthDayTime(g_statsAll.timeStart) << " ..... started STAR run\n" <<flush;
 
     //generate genome
-    if (P.runMode=="alignReads") 
+    if (P.runMode=="alignReads")
     {//continue
-    } else if (P.runMode=="genomeGenerate") 
+    } else if (P.runMode=="genomeGenerate")
     {
         Genome mainGenome(P);
         mainGenome.genomeGenerate();
         (void) sysRemoveDir (P.outFileTmp);
         P.inOut->logMain << "DONE: Genome generation, EXITING\n" << flush;
         exit(0);
-    } else if (P.runMode=="liftOver") 
-    {        
+    } else if (P.runMode=="liftOver")
+    {
         for (uint ii=0; ii<P.pGe.gChainFiles.size();ii++)
         {
             Chain chain(P,P.pGe.gChainFiles.at(ii));
@@ -93,17 +85,17 @@ int main(int argInN, char* argIn[]) {
         };
     } else {
         P.inOut->logMain << "EXITING because of INPUT ERROR: unknown value of input parameter runMode=" <<P.runMode<<endl<<flush;
-        exit(1);        
+        exit(1);
     };
 
     Genome mainGenome (P);
     mainGenome.genomeLoad();
-    
-    
+
+
     if (P.pGe.gLoad=="LoadAndExit" || P.pGe.gLoad=="Remove")
     {
         return 0;
-    };     
+    };
 
     //calculate genome-related parameters
     Transcriptome *mainTranscriptome=NULL;
@@ -133,6 +125,7 @@ int main(int argInN, char* argIn[]) {
         pthread_mutex_init(&g_threadChunks.mutexOutFilterBySJout, NULL);
         pthread_mutex_init(&g_threadChunks.mutexStats, NULL);
         pthread_mutex_init(&g_threadChunks.mutexBAMsortBins, NULL);
+        pthread_mutex_init(&g_threadChunks.mutexError, NULL);
     };
 
     g_statsAll.progressReportHeader(P.inOut->logProgress);
@@ -154,6 +147,8 @@ int main(int argInN, char* argIn[]) {
         P1.quant.yes=false;
         P1.quant.trSAM.yes=false;
         P1.quant.geCount.yes=false;
+        P1.outSAMunmapped.within=false;
+        P1.quant.trSAM.bamYes=false;
 
         P1.outFilterBySJoutStage=0;
 
@@ -165,7 +160,8 @@ int main(int argInN, char* argIn[]) {
 //         P1.inOut->logMain.open((P1.outFileNamePrefix + "Log.out").c_str());
 
         P1.wasp.outputMode="None"; //no WASP filtering on the 1st pass
-        
+        P1.pSolo.type=0; //no solo in the first pass
+
         g_statsAll.resetN();
         time(&g_statsAll.timeStartMap);
         P.inOut->logProgress << timeMonthDayTime(g_statsAll.timeStartMap) <<"\tStarted 1st pass mapping\n" <<flush;
@@ -241,7 +237,7 @@ int main(int argInN, char* argIn[]) {
             };
             extrastream.close();
         };
-        
+
         if (P.outSAMheaderPG.at(0)!="-") {
             samHeaderStream << P.outSAMheaderPG.at(0);
             for (uint ii=1;ii<P.outSAMheaderPG.size(); ii++) {
@@ -271,7 +267,7 @@ int main(int argInN, char* argIn[]) {
 
 
         samHeaderStream <<  "@CO\t" <<"user command line: " << P.commandLine <<"\n";
-        
+
         samHeaderStream << P.samHeaderExtra;
 
         if (P.outSAMheaderHD.at(0)!="-") {
@@ -298,7 +294,7 @@ int main(int argInN, char* argIn[]) {
 //                 outBAMwriteHeader(P.inOut->outBAMfileCoord,P.samHeader,mainGenome.chrName,mainGenome.chrLength);
 //             };
 
-        if ( P.quant.trSAM.yes ) {
+        if ( P.quant.trSAM.bamYes ) {
             samHeaderStream.str("");
             vector <uint> trlength;
             for (uint32 ii=0;ii<mainTranscriptome->trID.size();ii++) {
@@ -341,7 +337,6 @@ int main(int argInN, char* argIn[]) {
         outputSJ(RAchunk,P);//collapse novel junctions
         P.readFilesIndex=-1;
 
-
         P.outFilterBySJoutStage=2;
         if (P.outBAMcoord) {
             for (int it=0; it<P.runThreadN; it++) {//prepare the unmapped bin
@@ -366,13 +361,22 @@ int main(int argInN, char* argIn[]) {
         P.limitBAMsortRAM=mainGenome.nGenome+mainGenome.SA.lengthByte+mainGenome.SAi.lengthByte;
     };
 
+    time(&g_statsAll.timeFinishMap);
+    *P.inOut->logStdOut << timeMonthDayTime(g_statsAll.timeFinishMap) << " ..... finished mapping\n" <<flush;
+
     //no need for genome anymore, free the memory
     mainGenome.freeMemory();
 
-    if ( P.quant.geCount.yes )
-    {//output gene quantifications
-        for (int ichunk=1; ichunk<P.runThreadN; ichunk++)
-        {//sum counts from all chunks into 0th chunk
+    //aggregate output junctions
+    //collapse splice junctions from different threads/chunks, and output them
+    outputSJ(RAchunk,P);
+
+    //solo genes
+    Solo soloMain(RAchunk,P,*RAchunk[0]->chunkTr);//solo for genes
+    soloMain.processAndOutput();
+
+    if ( P.quant.geCount.yes ) {//output gene quantifications
+        for (int ichunk=1; ichunk<P.runThreadN; ichunk++) {//sum counts from all chunks into 0th chunk
             RAchunk[0]->chunkTr->quants->addQuants(*(RAchunk[ichunk]->chunkTr->quants));
         };
         RAchunk[0]->chunkTr->quantsOutput();
@@ -382,7 +386,6 @@ int main(int argInN, char* argIn[]) {
         RAchunk[0]->chunkFilesCat(P.inOut->outSAM, P.outFileTmp + "/Aligned.out.sam.chunk", g_threadChunks.chunkOutN);
     };
 
-
     if (P.outBAMcoord) {//sort BAM if needed
         *P.inOut->logStdOut << timeMonthDayTime() << " ..... started sorting BAM\n" <<flush;
         P.inOut->logMain << timeMonthDayTime() << " ..... started sorting BAM\n" <<flush;
@@ -403,57 +406,68 @@ int main(int argInN, char* argIn[]) {
             errOut <<"EXITING because of fatal ERROR: not enough memory for BAM sorting: \n";
             errOut <<"SOLUTION: re-run STAR with at least --limitBAMsortRAM " <<maxMem+1000000000;
             exitWithError(errOut.str(), std::cerr, P.inOut->logMain, EXIT_CODE_PARAMETER, P);
-        };
-
-
-        uint totalMem=0;
-//         P.inOut->logMain << "Started sorting BAM ..." <<endl;
-        #pragma omp parallel num_threads(P.outBAMsortingThreadNactual)
-        #pragma omp for schedule (dynamic,1)
-        for (uint32 ibin1=0; ibin1<nBins; ibin1++) {
-            uint32 ibin=nBins-1-ibin1;//reverse order to start with the last bin - unmapped reads
-
-            uint binN=0, binS=0;
-            for (int it=0; it<P.runThreadN; it++) {//collect sizes from threads
-                binN += RAchunk[it]->chunkOutBAMcoord->binTotalN[ibin];
-                binS += RAchunk[it]->chunkOutBAMcoord->binTotalBytes[ibin];
+        } else if(maxMem==0) {
+            P.inOut->logMain << "WARNING: nothing to sort - no output alignments" <<endl;
+            BGZF *bgzfOut;
+            bgzfOut=bgzf_open(P.outBAMfileCoordName.c_str(),("w"+to_string((long long) P.outBAMcompression)).c_str());
+            if (bgzfOut==NULL) {
+                ostringstream errOut;
+                errOut <<"EXITING because of fatal ERROR: could not open output bam file: " << P.outBAMfileCoordName << "\n";
+                errOut <<"SOLUTION: check that the disk is not full, increase the max number of open files with Linux command ulimit -n before running STAR";
+                exitWithError(errOut.str(), std::cerr, P.inOut->logMain, EXIT_CODE_PARAMETER, P);
             };
+            outBAMwriteHeader(bgzfOut,P.samHeaderSortedCoord,mainGenome.chrNameAll,mainGenome.chrLengthAll);
+            bgzf_close(bgzfOut);
+        } else {//sort
+            uint totalMem=0;
+            #pragma omp parallel num_threads(P.outBAMsortingThreadNactual)
+            #pragma omp for schedule (dynamic,1)
+            for (uint32 ibin1=0; ibin1<nBins; ibin1++) {
+                uint32 ibin=nBins-1-ibin1;//reverse order to start with the last bin - unmapped reads
 
-            if (binS==0) continue; //empty bin
-
-            if (ibin == nBins-1) {//last bin for unmapped reads
-                BAMbinSortUnmapped(ibin,P.runThreadN,P.outBAMsortTmpDir, P, mainGenome);
-            } else {
-            uint newMem=binS+binN*24;
-            bool boolWait=true;
-            while (boolWait) {
-                #pragma omp critical
-                if (totalMem+newMem < P.limitBAMsortRAM) {
-                    boolWait=false;
-                    totalMem+=newMem;
+                uint binN=0, binS=0;
+                for (int it=0; it<P.runThreadN; it++) {//collect sizes from threads
+                    binN += RAchunk[it]->chunkOutBAMcoord->binTotalN[ibin];
+                    binS += RAchunk[it]->chunkOutBAMcoord->binTotalBytes[ibin];
                 };
-                sleep(0.1);
-            };
-            BAMbinSortByCoordinate(ibin,binN,binS,P.runThreadN,P.outBAMsortTmpDir, P, mainGenome);
-            #pragma omp critical
-            totalMem-=newMem;//"release" RAM
-        };
-        };
-        //concatenate all BAM files, using bam_cat
-        char **bamBinNames = new char* [nBins];
-        vector <string> bamBinNamesV;
-        for (uint32 ibin=0; ibin<nBins; ibin++) {
 
-            bamBinNamesV.push_back(P.outBAMsortTmpDir+"/b"+std::to_string((uint) ibin));
-            struct stat buffer;
-            if (stat (bamBinNamesV.back().c_str(), &buffer) != 0) {//check if file exists
-                bamBinNamesV.pop_back();
+                if (binS==0) continue; //empty bin
+
+                if (ibin == nBins-1) {//last bin for unmapped reads
+                    BAMbinSortUnmapped(ibin,P.runThreadN,P.outBAMsortTmpDir, P, mainGenome);
+                } else {
+                    uint newMem=binS+binN*24;
+                    bool boolWait=true;
+                    while (boolWait) {
+                        #pragma omp critical
+                        if (totalMem+newMem < P.limitBAMsortRAM) {
+                            boolWait=false;
+                            totalMem+=newMem;
+                        };
+                        sleep(0.1);
+                    };
+                    BAMbinSortByCoordinate(ibin,binN,binS,P.runThreadN,P.outBAMsortTmpDir, P, mainGenome);
+                    #pragma omp critical
+                    totalMem-=newMem;//"release" RAM
+                };
             };
+
+            //concatenate all BAM files, using bam_cat
+            char **bamBinNames = new char* [nBins];
+            vector <string> bamBinNamesV;
+            for (uint32 ibin=0; ibin<nBins; ibin++) {
+
+                bamBinNamesV.push_back(P.outBAMsortTmpDir+"/b"+std::to_string((uint) ibin));
+                struct stat buffer;
+                if (stat (bamBinNamesV.back().c_str(), &buffer) != 0) {//check if file exists
+                    bamBinNamesV.pop_back();
+                };
+            };
+            for (uint32 ibin=0; ibin<bamBinNamesV.size(); ibin++) {
+                    bamBinNames[ibin] = (char*) bamBinNamesV.at(ibin).c_str();
+            };
+            bam_cat(bamBinNamesV.size(), bamBinNames, 0, P.outBAMfileCoordName.c_str());
         };
-        for (uint32 ibin=0; ibin<bamBinNamesV.size(); ibin++) {
-                bamBinNames[ibin] = (char*) bamBinNamesV.at(ibin).c_str();
-        };
-        bam_cat(bamBinNamesV.size(), bamBinNames, 0, P.outBAMfileCoordName.c_str());
     };
     //wiggle output
     if (P.outWigFlags.yes) {
@@ -463,12 +477,8 @@ int main(int argInN, char* argIn[]) {
         signalFromBAM(P.outBAMfileCoordName, wigOutFileNamePrefix, P);
     };
 
-    //aggregate output junctions
-    //collapse splice junctions from different threads/chunks, and output them
-    outputSJ(RAchunk,P);
-
     g_statsAll.writeLines(P.inOut->outChimJunction, P.pCh.outJunctionFormat, "#", STAR_VERSION + string("   ") + P.commandLine);
-    
+
     g_statsAll.progressReport(P.inOut->logProgress);
     P.inOut->logProgress  << "ALL DONE!\n"<<flush;
     P.inOut->logFinal.open((P.outFileNamePrefix + "Log.final.out").c_str());
@@ -476,12 +486,8 @@ int main(int argInN, char* argIn[]) {
     *P.inOut->logStdOut << timeMonthDayTime(g_statsAll.timeFinish) << " ..... finished successfully\n" <<flush;
 
     P.inOut->logMain  << "ALL DONE!\n" << flush;
-    if (P.outTmpKeep=="None")
-    {
+    if (P.outTmpKeep=="None") {
         sysRemoveDir (P.outFileTmp);
-    } else
-    {
-        //nothing to do
     };
 
     P.closeReadsFiles();//this will kill the readFilesCommand processes if necessary
