@@ -13,7 +13,7 @@ void revComplementNucleotides(char* ReadsIn, char* ReadsOut, uint Lread); //comp
 void revComplementNucleotides(string &seq);
 char nuclToNumBAM(char cc);
 void nuclPackBAM(char* ReadsIn, char* ReadsOut, uint Lread);
-char convertNt01234(const char R0);//transform sequence  from ACGT into 0-1-2-3-4 code    
+char convertNt01234(const char R0);//transform sequence  from ACGT into 0-1-2-3-4 code
 
 uint chrFind(uint, uint, uint*); // find chromosome from global locus
 uint localSearch(const char*, uint, const char*, uint, double); //local search to clip adapter
@@ -24,4 +24,7 @@ uint qualitySplit(char*, char*, uint, char, uint, uint, uint**);
 int32 convertNuclStrToInt32(const string S, uint32 &intOut);
 string convertNuclInt32toString(const uint32 nuclNum, const uint32 L);
 
+int64 convertNuclStrToInt64(const string S, uint64 &intOut);
+string convertNuclInt64toString(const uint64 nuclNum, const uint32 L);
+
 #endif
