@@ -1,31 +1,36 @@
+#include <cmath>
 #include "SoloReadFeature.h"
 #include "binarySearch2.h"
-#include <math.h>
+#include "serviceFuns.cpp"
+
 
 bool inputFeatureUmi(fstream *strIn, int32 featureType, uint32 &feature, uint32 &umi, array<vector<uint64>,2> &sjAll)
 {
     if (!(*strIn >> umi)) //end of file
         return false;
-    
-    if (featureType==0) {//gene
+
+    if (featureType==0 || featureType==2) {//gene
         *strIn >> feature;
     } else if (featureType==1) {//sj
         uint32 sj[2];
         *strIn >> sj[0] >> sj[1];
         feature=(uint32) binarySearch2(sj[0],sj[1],sjAll[0].data(),sjAll[1].data(),sjAll[0].size());
     };
-    
-    return true;       
+
+    return true;
 };
 
-void SoloReadFeature::inputRecords(uint32 **cbP, uint32 *cbReadCountExactTotal) 
+void SoloReadFeature::inputRecords(uint32 **cbP, uint32 *cbReadCountExactTotal)
 {
     {//load exact matches
         strU_0->flush();
         strU_0->seekg(0,ios::beg);
-        uint32 cb, feature, umi;
+        uint32 feature, umi;
+        int64 cb;
         while (inputFeatureUmi(strU_0, featureType, feature, umi, P.sjAll)) {
             *strU_0 >> cb;
+            if (!pSolo.cbWLyes) //if no-WL, the full cbInteger was recorded - now has to be placed in order
+                cb=binarySearchExact<uint64>(cb,pSolo.cbWL.data(),pSolo.cbWL.size());
             if (feature != (uint32)(-1)){
                 cbP[cb][0]=feature;
                 cbP[cb][1]=umi;
@@ -34,6 +39,9 @@ void SoloReadFeature::inputRecords(uint32 **cbP, uint32 *cbReadCountExactTotal)
             };
         };
     };
+
+    if (!pSolo.cbWLyes) //no WL => no mismatch check
+        return;
     
     {//1 match
         strU_1->flush();
@@ -52,7 +60,7 @@ void SoloReadFeature::inputRecords(uint32 **cbP, uint32 *cbReadCountExactTotal)
             };
         };
     };
-     
+
     {//2 matches
         strU_2->flush();
         strU_2->seekg(0,ios::beg);
@@ -72,7 +80,7 @@ void SoloReadFeature::inputRecords(uint32 **cbP, uint32 *cbReadCountExactTotal)
                 if (cbReadCountExactTotal[cbin]>0) {//otherwise this cbin does not work
                     qin -= pSolo.QSbase;
                     qin = qin < pSolo.QSmax ? qin : pSolo.QSmax;
-                    pin=cbReadCountExactTotal[cbin]*pow(10.0,-qin/10.0);
+                    pin=cbReadCountExactTotal[cbin]*std::pow(10.0,-qin/10.0);
                     ptot+=pin;
                     if (pin>pmax) {
                         cb=cbin;
@@ -86,7 +94,7 @@ void SoloReadFeature::inputRecords(uint32 **cbP, uint32 *cbReadCountExactTotal)
                 cbP[cb]+=2;
             } else {
                 stats.V[stats.nTooMany]++;
-            };        
+            };
         };
     };
 };
