@@ -32,14 +32,21 @@ public:
        uint32 *g, *t; //gene/transcript IDs
     } exG;
 
+    struct {//geneFull structure
+        uint64 *s, *e, *eMax;
+        uint8 *str;
+        uint32 *g;
+    } geneFull;
+
     Quantifications *quants;
-    
+
     //methods:
     Transcriptome (Parameters &Pin); //create transcriptome structure, load and initialize parameters
-    uint32 quantAlign (Transcript &aG, Transcript *aTall, vector<uint32> &readTranscripts, set<uint32> &readTrGenes);//transform coordinates for all aligns from genomic in RA to transcriptomic in RAtr
+    uint32 quantAlign (Transcript &aG, Transcript *aTall, vector<uint32> &readTranscripts, set<uint32> &readGene);//transform coordinates for all aligns from genomic in RA to transcriptomic in RAtr
     void geneCountsAddAlign(uint nA, Transcript **aAll, vector<int32> &gene1); //add one alignment to gene counts
     void quantsAllocate(); //allocate quants structure
-    void quantsOutput(); //output quantification files   
+    void quantsOutput(); //output quantification files
+    void geneFullAlignOverlap(uint nA, Transcript **aAll, int32 strandType, set<uint32> &geneOverlap);
 
 private:
     Parameters &P; //normal "genomic" parameters
