@@ -1,9 +1,9 @@
 #include "stitchWindowAligns.h"
-#include <math.h>
-#include <time.h>
 #include "blocksOverlap.h"
 #include "ErrorWarning.h"
 #include "binarySearch2.h"
+#include <cmath>
+#include <ctime>
 
 void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, uint tG2, Transcript trA, \
                         uint Lread, uiWA* WA, char* R, Genome &mapGen, \
@@ -80,7 +80,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
         };
         };
 
-        if (P.alignSoftClipAtReferenceEnds=="No" &&  \
+        if (!P.alignSoftClipAtReferenceEnds.yes &&  \
                 ( (trA.exons[trA.nExons-1][EX_G] + Lread-trA.exons[trA.nExons-1][EX_R]) > (mapGen.chrStart[trA.Chr]+mapGen.chrLength[trA.Chr]) || \
                    trA.exons[0][EX_G]<(mapGen.chrStart[trA.Chr]+trA.exons[0][EX_R]) ) ) {
             return; //no soft clipping past the ends of the chromosome
@@ -109,9 +109,9 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
 
         //filter strand consistency
         uint sjN=0;
-        trA.intronMotifs[0]=0;trA.intronMotifs[1]=0;trA.intronMotifs[2]=0;        
+        trA.intronMotifs[0]=0;trA.intronMotifs[1]=0;trA.intronMotifs[2]=0;
         for (uint iex=0;iex<trA.nExons-1;iex++) {
-            if (trA.canonSJ[iex]>=0) 
+            if (trA.canonSJ[iex]>=0)
             {//junctions - others are indels
                 sjN++;
                 trA.intronMotifs[trA.sjStr[iex]]++;
@@ -128,7 +128,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
         if (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0 && P.outFilterIntronStrands=="RemoveInconsistentStrands")
                 return;
 
-        if (sjN>0 && trA.sjMotifStrand==0 && P.outSAMstrandField=="intronMotif") {//strand not defined for a junction
+        if (sjN>0 && trA.sjMotifStrand==0 && P.outSAMstrandField.type==1) {//strand not defined for a junction
             return;
         };
 
@@ -189,8 +189,8 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
                 if (trA.exons[0][EX_G] > \
                     trA.exons[iexM2][EX_G]+trA.exons[0][EX_R]+P.alignEndsProtrude.nBasesMax) return; //LeftMateStart > RightMateStart + allowance
                 if (trA.exons[iexM2-1][EX_G]+trA.exons[iexM2-1][EX_L] > \
-                   trA.exons[trA.nExons-1][EX_G]+Lread-trA.exons[trA.nExons-1][EX_R]+P.alignEndsProtrude.nBasesMax) return; //LeftMateEnd   > RightMateEnd +allowance            
-                
+                   trA.exons[trA.nExons-1][EX_G]+Lread-trA.exons[trA.nExons-1][EX_R]+P.alignEndsProtrude.nBasesMax) return; //LeftMateEnd   > RightMateEnd +allowance
+
                 //check for junctions consistency
                 uint iex1=1, iex2=iexM2+1; //last exons of the junction
                 for  (; iex1<iexM2; iex1++) {//find first junction that overlaps 2nd mate
@@ -236,9 +236,9 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
 
         //Variation
         Score+=trA.variationAdjust(mapGen, R);
-        
+
         trA.maxScore=Score;
-        
+
         // transcript has been finalized, compare the score and record
         if (       Score+P.outFilterMultimapScoreRange >= wTr[0]->maxScore \
                 || ( trA.iFrag>=0 && Score+P.outFilterMultimapScoreRange >= RA->maxScoreMate[trA.iFrag] ) \
